library rapidfire;

//import 'package:flushbar/flushbar.dart';
import 'package:rapidfire/vendor/flushbar.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_statusbarcolor/flutter_statusbarcolor.dart';
import 'package:page_transition/page_transition.dart';
import 'package:rapidfire/AccessControl.dart';
import 'package:rapidfire/models/AppUser.dart';
import 'package:rapidfire/Api.dart';
import 'package:rapidfire/uiux/StyleRF.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'Config.dart';
import 'Db.dart';
import 'Dynamic.dart';
import 'GlobalRF.dart';
import 'LocalizationRF.dart';
import 'GlobalScopeRF.dart';
import 'GlobalScopeRF.dart' as g;
import 'Rdb.dart';
import 'RestartRF.dart';
import 'package:catcher/catcher.dart';
import 'SystemRF.dart';
import 'Const.dart';

export 'package:flutter_speed_dial/flutter_speed_dial.dart';
export 'Config.dart';
export 'Dynamic.dart';
export 'UtilRF.dart';
export 'package:rapidfire/GlobalRF.dart';
export 'package:rapidfire/uiux/StyleRF.dart';
export 'package:rapidfire/uiux/WidgetsRF.dart';
export 'package:rapidfire/Db.dart';
export 'package:jiffy/jiffy.dart';
export 'package:intl/intl.dart';
export 'package:rapidfire/StockRF.dart';
export 'package:rapidfire/uiux/DrawerRF.dart';
export 'package:rapidfire/Api.dart';
export 'package:rapidfire/uiux/RipplesRF.dart';
export 'package:rapidfire/Extensions.dart';
export 'package:rapidfire/Const.dart';
export 'package:rapidfire/BGSyncSMain.dart';
export 'package:workmanager/workmanager.dart';

enum MessageType { info, error, warning }

abstract class IJobWorkerRF {
  execute(String taskName);
}

class RapidFire {
  Widget _startPage;
  bool _handelGlobalError;
  AccessControl accessControl;
  IConfigRF _config;
  Configuration _configuration = new Configuration();
  SharedPreferences pref;
  App app;
  ModelObject model; //????
  Map<String, dynamic> models;
  Res lang;
  Db db;
  Rdb rdb;
  Api api;
  Mapper mapper;
  AppStyles style;
  AppOnboardRF onboard;
  PageAction page;
  AppUser appUser;

  BuildContext context;
  IJobWorkerRF worker;

  static Catcher catcher;
  CatcherOptions debugOptions, releaseOptions;

  RapidFire(IConfigRF configuration, Function reflectionAgent, {Widget startupPage, bool globalErrorEnabled = true}) {
    if (reflectionAgent != null) reflectionAgent.call();

    WidgetsFlutterBinding.ensureInitialized();

    _config = configuration;
    _startPage = startupPage;
    _handelGlobalError = globalErrorEnabled;

    if (_handelGlobalError) {
      run(() {
        if (globalErrorEnabled) {
          var explicitMap = {"Error": ConsoleHandler()};
          debugOptions = CatcherOptions(
            DialogReportMode(),
            [
              ConsoleHandler(
                enableApplicationParameters: false,
                enableCustomParameters: false,
                enableStackTrace: true,
                enableDeviceParameters: false,
              )
            ],
            explicitExceptionHandlersMap: explicitMap,
          );
          releaseOptions = CatcherOptions(DialogReportMode(), [
            EmailManualHandler(["a@b.com"]),
          ]);
          catcher = Catcher(
            rootWidget: RestartRF(RapidFireAppRF(_startPage ?? onboard.next())),
            debugConfig: debugOptions,
            releaseConfig: releaseOptions,
          );
        }
      });
    }
  }

  Future<dynamic> run([dynamic callback]) async {
    rf = this;

    pref = await SharedPreferences.getInstance();

    _config.configure(_configuration);
    _configuration.model.appModels.register(g.modelList); //==>>>>
    Libs.config = _configuration;

    accessControl = AccessControl();
    db = Db();
    rdb = Rdb();
    api = Api();
    app = App();
    lang = Res();
    mapper = Mapper();
    style = AppStyles();
    page = PageAction();
    models = Libs.appModels.get();

    if (AppOnboardRF.currentStep == 0) {
      onboard = AppOnboardRF(
        splash: AppOnboardRF.addItem(_configuration, OnboardItemsRF.splash),
        language: AppOnboardRF.addItem(_configuration, OnboardItemsRF.language),
        intro: AppOnboardRF.addItem(_configuration, OnboardItemsRF.intro),
        login: AppOnboardRF.addItem(_configuration, OnboardItemsRF.login),
        otp: AppOnboardRF.addItem(_configuration, OnboardItemsRF.otp),
        initialSetup: AppOnboardRF.addItem(_configuration, OnboardItemsRF.setup),
        home: AppOnboardRF.addItem(_configuration, OnboardItemsRF.home),
      );
      AppOnboardRF.loadOnboardItems(onboard);
    }

    appUser = AppUser();
    appUser.Name = 'Anawer Hossain';
    Libs.appUser = appUser;

    Libs.msg = msg;

    _configuration.app.init?.forEach((element) => element.call());

    if (callback.toString().contains('Closure: () => Null')) {
      if (_handelGlobalError) {
        callback();
      }
    } else {
      if (!_handelGlobalError) {
        callback(RestartRF(RapidFireAppRF(_startPage ?? onboard.next())));
      }
    }

    await pref.setString(ConstRF.dbName, _configuration.db.name);
    await pref.setInt(ConstRF.dbVersion, _configuration.db.version);

    await rf.db.execute('update Beneficiary set UserId = 240');
  }

  static updateGlobalErrorLanguage() {
    var explicitMap = {"Error": ConsoleHandler()};
    if (catcher != null) {
      catcher.updateConfig(
        debugConfig: CatcherOptions(
          DialogReportMode(),
          [
            ConsoleHandler(
              enableApplicationParameters: false,
              enableCustomParameters: false,
              enableStackTrace: true,
              enableDeviceParameters: false,
            )
          ],
          localizationOptions: [globalErrorMessage()],
          explicitExceptionHandlersMap: explicitMap,
        ),
      );
    }
  }

  static globalErrorMessage() {
    return LocalizationOptions(
      "en",
      dialogReportModeTitle: rf.lang.text('System', 'dialogReportModeTitle'),
      dialogReportModeDescription: rf.lang.text('System', 'dialogReportModeDescription'),
      dialogReportModeAccept: rf.lang.text('System', 'dialogReportModeAccept'),
      dialogReportModeCancel: rf.lang.text('System', 'dialogReportModeCancel'),
    );
  }

  Flushbar fb;

  msg(BuildContext context, String text,
      {MessageType type = MessageType.info,
      IconData icon,
      Color color = Colors.black87,
      Color backgroundColor = Colors.redAccent,
      String actionText = '',
      Function action}) {
    fb = Flushbar<bool>(
      //title:  null,
      messageText: Padding(
        padding: const EdgeInsets.only(left: 20),
        child: TextRF(
          text,
          style: TextStyle(fontWeight: FontWeight.w500, fontSize: 18.0, color: color, fontFamily: "ShadowsIntoLightTwo"),
        ),
      ),
      duration: Duration(seconds: 2),
      margin: EdgeInsets.all(10),
      padding: EdgeInsets.all(8),
      borderRadius: 50,
      backgroundColor: type == MessageType.info
          ? Colors.grey[300]
          : type == MessageType.warning
              ? Colors.deepOrange
              : type == MessageType.error
                  ? Colors.red
                  : Colors.lightBlueAccent,
      icon: Icon(
        icon ?? Icons.info,
        color: Colors.white,
        size: 52,
      ),
      mainButton: actionText.isNotEmpty
          ? Padding(
              padding: const EdgeInsets.only(right: 6.0),
              child: FlatButton(
                shape: new RoundedRectangleBorder(borderRadius: new BorderRadius.circular(50.0)),
                color: Colors.white,
                onPressed: () {
                  fb.dismiss(true);
                },
                child: Text(
                  actionText,
                  style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold, fontSize: 18.0),
                ),
              ),
            )
          : null,
    )..show(context).then((value) {
        if (action != null) if (value != null) if (value) action();
      });
  }
}

class PageAction {
  Future call(BuildContext context, dynamic pageToCall, {bool refreshCaller = false, TransitionTypeRF pageTransition}) async {
    var callerStatusBarColor = await FlutterStatusbarcolor.getStatusBarColor();
    var callerNavigationBarColor = await FlutterStatusbarcolor.getNavigationBarColor();

    PageTransitionType getPageTransitionType() {
      PageTransitionType type;
      if (pageTransition == null) {
        if (Libs.config.page.transition == null) {
          type = PageTransitionType.bottomToTop;
        } else {
          type = PageTransitionType.values[Libs.config.page.transition.index];
        }
      } else {
        type = PageTransitionType.values[pageTransition.index];
      }
      return type;
    }

    //PageRF.parentContext = pageToCall.toString();
    var callerClassName = context;
    PageRF.currentPageClassName = pageToCall.toString();

    if (!PageRF.callerStack.contains(pageToCall.toString())) {
      PageRF.callerStack.push(pageToCall.toString());
    }

    return Navigator.push(
      context,
      PageTransition(
        curve: Curves.linear,
        type: getPageTransitionType(),
        child: pageToCall,
      ),
    ).then((value) async {
      SystemRF.os.setOsBarColors([callerStatusBarColor, callerNavigationBarColor]);
      if (refreshCaller) {
        if (PageRF.onCallerRefreshMap != null) {
          var className = callerClassName.widget.toString();
          var func = PageRF.onCallerRefreshMap[className];
          if (func != null) {
            await func();
          }
        }

        if (PageRF.dataSourcesCallerRefreshMap != null) {
          var className = callerClassName.widget.toString();
          var dsList = PageRF.dataSourcesCallerRefreshMap[className];
          if (dsList != null) for (var ds in dsList) ds.refresh();
        }
      }
      PageRF.currentPageClassName = callerClassName;
    });

    //    return Navigator.push(context, new MaterialPageRoute(builder: (context) => page)).then((value) {
    //      if (refresh) PageRF.refresh(context);
    //    });

    //    return Navigator.of(context).push(_createRoute(page)).then((value) {
    //      if (refresh) PageRF.refresh(context);
    //    });
  }

  back(BuildContext context, {model}) {
    //    Navigator.pushAndRemoveUntil(context,MaterialPageRoute(builder: (context) => page),ModalRoute.withName("/Home"));
    //    return Navigator.pushReplacement(context, new MaterialPageRoute(builder: (context) => page),);
    //    Navigator.pop(context);

    PageRF.willPop(context, model: model);
  }

  replace(BuildContext context, dynamic page) {
    return Navigator.pushReplacement(
      context,
      new MaterialPageRoute(builder: (context) => page),
    );
  }

  Route _createRoute(dynamic page) {
    return PageRouteBuilder(
      pageBuilder: (context, animation, secondaryAnimation) => page,
      transitionDuration: Duration(milliseconds: 250),
      maintainState: false,
      transitionsBuilder: (context, animation, secondaryAnimation, child) {
        var begin = Offset(0.0, 1.0);
        var end = Offset.zero;
        var curve = Curves.ease;

        var tween = Tween(begin: begin, end: end).chain(CurveTween(curve: curve));

        return SlideTransition(
          position: animation.drive(tween),
          child: child,
        );
      },
    );
  }
}

class ModelObject {
  dynamic _model;

  ModelObject(dynamic model) {
    _model = model;
  }

  bool changed() {
    return Dynamic(_model).changed1();
  }
}

class Res {
  String text(String model, String name) {
    return LocalizationRF.instance.text(model, name);
  }
}

class App {
  void restart(BuildContext context) {
    RestartRF.restartApp(context);
  }
}

class Mapper {
  Object jsonToModel(Object obj, Map<String, dynamic> json) {
    Dynamic dynamic = Dynamic(obj);
    dynamic.dataMapToModel(json);
    return obj;
  }
}

class ScreenRF {
  static height(BuildContext context) {
    return MediaQuery.of(context).size.height;
  }

  static width(BuildContext context) {
    return MediaQuery.of(context).size.width;
  }
}

class ClassRF {
  copy() {}

  compare(Object comparableObject) {}
}
