//import 'package:flushbar/flushbar.dart';
import 'package:rapidfire/vendor/flushbar.dart';
import 'package:flutter/material.dart';
import 'package:flutter_custom_clippers/flutter_custom_clippers.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:rapidfire/stock/ref/main.dart';
import 'package:rapidfire/uiux/DrawerRF.dart';
import 'package:rapidfire/uiux/WidgetStylesRF.dart';
import 'package:rapidfire/stock/modern/defaults/AppLayout.dart';
import 'package:rapidfire/stock/modern/onboarding/auth_page.dart';

import 'Config.dart';
import 'GlobalRF.dart';
import 'GlobalScopeRF.dart';
import 'uiux/StyleRF.dart';
import 'package:rapidfire/stock/modern/onboarding/HomeRF.dart' as modern;
import 'package:rapidfire/stock/modern/onboarding/Splash.dart' as modern;
import 'package:rapidfire/stock/modern/onboarding/Login.dart' as modern;
import 'package:rapidfire/stock/modern/onboarding/Otp.dart' as modern;
import 'package:rapidfire/stock/modern/onboarding/Setup.dart' as modern;
import 'package:rapidfire/stock/modern/onboarding/Language.dart' as modern;
import 'package:rapidfire/stock/modern/onboarding/Language.dart';
import 'package:rapidfire/stock/modern/onboarding/InternetUsage.dart';

import 'uiux/WidgetsRF.dart';

class StockRF {
  static Onboard onboard = Onboard();
  static Outline outline = Outline(); //Outline -> Drawer, type selection

  static ImplementationStoreDefaults implementation(TemplateRF template) {
    dynamic _scaffold;
    ImplementationStoreDefaults implementationStoreDefaults = ImplementationStoreDefaults();

    template = TemplateRF.get(template);
    if (template.name == TemplateNameRF.modern) _scaffold = AppLayout();
    if (template.name == TemplateNameRF.classic) _scaffold = AppLayout();
    implementationStoreDefaults.page = PageImplementationStoreDefaults(outline: _scaffold as IPageOutlineRF);
    return implementationStoreDefaults;
  }

  static ScaffoldStoreDefaults scaffold({TemplateRF template, BuildContext context, dynamic model, Function onTap}) {
    if (template == null) template = TemplateRF.get();
    ScaffoldStoreDefaults scaffoldStoreDefaults = ScaffoldStoreDefaults();

    scaffoldStoreDefaults.outline = ScaffoldStoreDefaultsOutline(template); //drawerItems
    scaffoldStoreDefaults.app = ScaffoldStoreDefaultsApp(template); //modern ui, validated pop
    scaffoldStoreDefaults.widgets =
        ScaffoldStoreDefaultsWidgets(template: template, model: model, onTap: onTap); //tab bar
    scaffoldStoreDefaults.widgets.tabBar = ScaffoldStoreDefaultsWidgetsTabBar(template); //tab bar

    return scaffoldStoreDefaults;
  }

  static StyleStoreDefaults style(TemplateRF template) {
    dynamic _styles;

    template = TemplateRF.get(template);
    if (template.name == TemplateNameRF.classic) _styles = AppLayout();
    if (template.name == TemplateNameRF.modern) _styles = AppLayout(); //classic class

    return StyleStoreDefaults(text: _styles as ITextRFStyle, container: _styles);
  }
}

class ImplementationStoreDefaults {
  PageImplementationStoreDefaults page;
}

class ScaffoldStoreDefaults {
  ScaffoldStoreDefaultsOutline outline;
  ScaffoldStoreDefaultsApp app;
  ScaffoldStoreDefaultsWidgets widgets;
}

class ScaffoldStoreDefaultsWidgets {
  TemplateRF _template;
  BuildContext _context;
  dynamic _model;
  Function _onTap;

  ScaffoldStoreDefaultsWidgets({TemplateRF template, BuildContext context, dynamic model, Function onTap}) {
    if (template == null) template = TemplateRF.get();
    _template = template;
    _context = context;
    _onTap = onTap;
  }

  get leading {
    if (_template.name == TemplateNameRF.modern && _template.version.contains('iOS')) {
      return InkWell(
        onTap: () => _onTap(), //rf.page.back(_context),
        child: Icon(Icons.arrow_back_ios, color: Colors.black),
      );
    }
  }

  ScaffoldStoreDefaultsWidgetsTabBar tabBar;
}

class ScaffoldStoreDefaultsWidgetsTabBar {
  TemplateRF _template;

  ScaffoldStoreDefaultsWidgetsTabBar([TemplateRF template]) {
    template = TemplateRF.get();
    _template = template;
  }

  getStyle(int index, int currentIndex, {String label, Icon icon, double labelWidth = 150}) {
    return Container(
      width: labelWidth,
      decoration: BoxDecoration(
        color: currentIndex == index ? Colors.white : Colors.transparent,
        borderRadius: BorderRadius.circular(50),
        border: Border.all(
          color: currentIndex == index ? Colors.white : Colors.white,
          width: currentIndex == index ? 0 : 2,
        ),
      ),
      child: Row(
        children: <Widget>[
          Padding(
            padding: EdgeInsets.only(
              top: currentIndex == index ? 3 : 2,
              bottom: currentIndex == index ? 2 : 2.5,
              left: currentIndex == index ? 3 : 0,
            ),
            child: CircleAvatar(
              child: Text(
                (index + 1).toString(),
                style: TextStyle(
                    fontSize: 20,
                    fontWeight: FontWeight.bold,
                    color: currentIndex == index ? Colors.white : Colors.blue),
              ),
              radius: 18,
              backgroundColor: currentIndex == index ? Colors.blue : Colors.white,
              foregroundColor: currentIndex == index ? Colors.white : Colors.blue,
            ),
          ),
          Expanded(
            child: Padding(
              padding: const EdgeInsets.only(top: 2.0),
              child: Align(
                alignment: Alignment.center,
                child: Text(
                  label,
                  style: TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: 16,
                      color: currentIndex == index ? Colors.blue : Colors.white),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class StyleStoreDefaults {
  final dynamic everything;
  final ITextRFStyle text;
  final IContainerRFStyle container;

  const StyleStoreDefaults({this.everything, this.text, this.container});
}

class Onboard {
  splash(Configuration _configuration, {TemplateRF template}) {
    var splash;
    switch (TemplateRF.get(template).name) {
      case TemplateNameRF.modern:
      case TemplateNameRF.material:
        if (_configuration.app.onboard is AppOnboardRF) {
          splash = _configuration.app.onboard?.splash ?? modern.Splash();
        } else {
          splash = _configuration.app.onboard?.customOnboard()?.splash ?? modern.Splash();
        }
        break;
    }
    return splash;
  }

  language(Configuration _configuration, {TemplateRF template}) {
    var language;
    switch (TemplateRF.get(template).name) {
      case TemplateNameRF.modern:
      case TemplateNameRF.material:
        if (_configuration.app.onboard is AppOnboardRF) {
          language = _configuration.app.onboard?.language ?? modern.LanguageSelection(isOnboarding: false);
        } else {
          language =
              _configuration.app.onboard?.customOnboard()?.language ?? modern.LanguageSelection(isOnboarding: false);
        }
        break;
    }
    return language;
  }

  intro(Configuration _configuration, {TemplateRF template}) {
    //'Not Implemented'
  }

  login(Configuration _configuration, {TemplateRF template}) {
    var login;
    switch (TemplateRF.get(template).name) {
      case TemplateNameRF.modern:
      case TemplateNameRF.material:
        if (_configuration.app.onboard is AppOnboardRF) {
          login = _configuration.app.onboard?.login ?? modern.Login();
        } else {
          login = _configuration.app.onboard?.customOnboard()?.login ?? modern.Login();
        }
        break;
    }
    return login;
  }

  otp(Configuration _configuration, {TemplateRF template}) {
    var otp;
    switch (TemplateRF.get(template).name) {
      case TemplateNameRF.modern:
      case TemplateNameRF.material:
        if (_configuration.app.onboard is AppOnboardRF) {
          otp = _configuration.app.onboard?.otp ?? modern.Otp();
        } else {
          otp = _configuration.app.onboard?.customOnboard()?.otp ?? modern.Otp();
        }
        break;
    }
    return otp;
  }

  setup(Configuration _configuration, {TemplateRF template}) {
    var setup;
    switch (TemplateRF.get(template).name) {
      case TemplateNameRF.modern:
      case TemplateNameRF.material:
        if (_configuration.app.onboard is AppOnboardRF) {
          setup = _configuration.app.onboard?.initialSetup ?? modern.Setup();
        } else {
          setup = _configuration.app.onboard?.customOnboard()?.initialSetup ?? modern.Setup();
        }
        break;
    }
    return setup;
  }
}

// type of drawer selection
class Outline {
  drawer(
    BuildContext context,
    String title,
    String subTitle,
    String imagePath,
    Color circleColor, [
    TemplateRF template,
  ]) {
    var drawerModernOval = DrawerRF(
        clipper: OvalRightBorderClipper(),
        menuHeader: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            Container(
              color: Colors.transparent,
              height: 240,
              padding: EdgeInsets.only(top: 15),
              child: ScaffoldStoreDefaultsOutline()
                  .drawerHeader(title, subTitle, imagePath, circleColor, template: template),
            )
          ],
        ),
        menuBody: ScaffoldStoreDefaultsOutline()
            .drawerBodyItems(context, separator: true, textWeight: FontWeight.bold, template: template),
        menuFooter: ScaffoldStoreDefaultsOutline()
            .drawerFooter(context, color: Colors.pink, separator: false, template: template)
//      DrawerItemRF(
//        icon: FontAwesomeIcons.headphonesAlt,
//        text: 'Support',
//        color: Colors.pink,
//        separator: false,
//        template: template,
//      ),
        );

    var drawerMaterialDefault = DrawerRF(
      menuHeader: Container(
        color: Colors.red,
        child: ScaffoldStoreDefaultsOutline().drawerHeader(title, subTitle, imagePath, circleColor, template: template),
//        DrawerHeader(
//          child: Column(
//            crossAxisAlignment: CrossAxisAlignment.stretch,
//            mainAxisAlignment: MainAxisAlignment.start,
//            children: <Widget>[
//              Padding(
//                padding: const EdgeInsets.only(right: 8.0),
//                child: CircleAvatar(
//                  radius: 45,
//                  backgroundColor: Color(0xffFDCF09),
//                  child: CircleAvatar(
//                      radius: 40, backgroundImage: AssetImage('assets/images/profile.png')),
//                ),
//              ),
//              SizedBox(height: 5),
//              Text(
//                'Drawer Header',
//                style: TextStyle(fontWeight: FontWeight.bold, color: Colors.white, fontSize: 18),
//              ),
//              Text('Sonething', style: TextStyle(color: Colors.white)),
//            ],
//          ),
//          decoration: BoxDecoration(color: Colors.blue),
//        ),
      ),
      menuBody: ScaffoldStoreDefaultsOutline()
          .drawerBodyItems(context, separator: false, textWeight: FontWeight.w500, template: template),
      menuFooter: ScaffoldStoreDefaultsOutline()
          .drawerFooter(context, separator: false, padding: 10, color: Colors.pink, template: template),
//      DrawerItemRF(
//          icon: FontAwesomeIcons.headphonesAlt,
//          text: 'Support',
//          color: Colors.pink,
//          separator: false,
//          template: template),
    );

    template = TemplateRF.get(template);
    if (template.name == TemplateNameRF.modern && template.version == 'Oval') return drawerModernOval;
    if (template.name == TemplateNameRF.material) return drawerMaterialDefault;
  }
}

enum ActionItemTypeRF { geo, floating, sci }

//https://buildflutter.com/functional-programming-with-flutter/
class ScaffoldStoreDefaultsOutline {
  TemplateRF _template;

  ScaffoldStoreDefaultsOutline([TemplateRF template]) {
    template = TemplateRF.get();
    _template = template;
  }

  drawerHeader(
    String title,
    String subTitle,
    String imagePath,
    Color circleColor, {
    TemplateRF template,
  }) {
    var modernHeader = Container(
      height: 80,
      width: 250,
      child: Stack(children: <Widget>[
        Text(title, style: StyleRF.theme.appTitle(template)),
        Positioned(
            top: 35,
            child: Text(
              subTitle,
              style: StyleRF.theme.appSubTitle(template),
            ))
      ]),
    );

    var modernOvalHeader = Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        Container(
          color: Colors.transparent,
          height: 240,
          padding: EdgeInsets.only(top: 15),
          child: Container(
            padding: EdgeInsets.zero,
            child: Stack(
              alignment: AlignmentDirectional.center,
              children: <Widget>[
                Positioned(
                  child: FaIcon(FontAwesomeIcons.powerOff, color: Colors.red),
                  right: 50,
                  top: 10,
                ),
                Positioned(
                    top: 30.0,
                    left: 105,
                    child: Column(
                      children: <Widget>[
                        SizedBox(height: 20),
                        CircleAvatar(
                          radius: 55,
                          backgroundColor: Colors.redAccent,
                          child: CircleAvatar(
                            radius: 50,
                            backgroundImage: AssetImage(imagePath), //
                          ),
                        ),
                        Padding(
                          padding: EdgeInsets.only(top: 5),
                          child: Text(
                            title,
                            style: TextStyle(
                              fontWeight: FontWeight.bold,
                              color: Colors.black,
                              fontSize: 18,
                            ),
                          ),
                        ),
                        Padding(
                          padding: const EdgeInsets.only(top: 0),
                          child: Text(
                            subTitle,
                            style: TextStyle(color: Colors.black),
                          ),
                        ),
                      ],
                    )),
              ],
            ),
          ),
        )
      ],
    );

    var materialHeader = Container(
      height: 250,
      color: Colors.blue,
      alignment: Alignment(0, 0),
      child: Column(
        children: <Widget>[
          SizedBox(height: 60),
          CircleAvatar(
            radius: 55,
            backgroundColor: Colors.redAccent,
            child: CircleAvatar(
              radius: 50, backgroundImage: AssetImage(imagePath), //
            ),
          ),
          SizedBox(height: 10),
          Text(
            title,
            style: StyleRF.theme.pageTitle(TemplateRF(null, style: StyleRF(textColor: Colors.white))),
          ),
          //.theme.pageTitle()
          Text(
            subTitle,
            style: StyleRF.theme.pageSubTitle(TemplateRF(null, style: StyleRF(textColor: Colors.white, size: 16))),
          )
        ],
      ),
    );

    template = TemplateRF.get(template);
    if (template.name == TemplateNameRF.modern && template.version == 'Oval')
      return modernOvalHeader;
    else if (template.name == TemplateNameRF.modern)
      return modernHeader;
    else if (template.name == TemplateNameRF.material) return materialHeader;
  }

  drawerMenuButton(Function onTap, {dynamic template}) {
    return IconButton(
      icon: Icon(Icons.menu),
      onPressed: () => onTap(),
      color: Colors.black54,
    );
  }

  pageHeaderActions(List<ActionItemsRF> actionItems, {List<PopupMenuItemsRF> popupMenuItems, TemplateRF template}) {
    template = TemplateRF.get(template);

    popupMenu(BuildContext context) {
      List<Widget> items = List();
      for (var item in popupMenuItems) {
        items.add(
          InkWell(
            onTap: () {
              item.action.call();
            },
            child: ListTile(
              leading: IconButton(icon: Icon(item.icon, color: Colors.blue, size: 26)),
              title: Text(item.label, style: TextStyle(fontSize: 18)),
            ),
          ),
        );
      }
      return items;
    }

    if (template.name == TemplateNameRF.modern) {
      List<Widget> widgets = List();
      for (var item in actionItems) {
        if (item.actionType == ActionItemTypeRF.geo) {
          widgets.addAll([
            IconButton(icon: Icon(Icons.my_location, color: Colors.black45), onPressed: item.action),
          ]);
        }
        if (item.actionType == null) {
          widgets.addAll([
            IconButton(icon: Icon(item.icon, color: Colors.black45), onPressed: item.action),
          ]);
        }
      }
      return widgets;
    }
    if (template.name == TemplateNameRF.material) {
      List<Widget> widgets = List();
      for (var item in actionItems) {
        if (item.actionType == ActionItemTypeRF.geo) {
          widgets.addAll([
            IconButton(icon: Icon(Icons.my_location, color: Colors.white), onPressed: item.action),
          ]);
        } else if (item.actionType == ActionItemTypeRF.sci) {
          widgets.addAll([
            IconButton(icon: Icon(Icons.my_location, color: Colors.white), onPressed: item.action),
            PopupMenuButton(
              itemBuilder: (BuildContext context) {
                return [PopupMenuItem(child: Column(children: popupMenu(context)))];
              },
            )
          ]);
        }
      }
      return widgets;
    }
  }

  bottomBar(int _selectedIndex, Function _onItemTapped, {TemplateRF template}) {
    return BottomNavigationBar(
      items: const <BottomNavigationBarItem>[
        BottomNavigationBarItem(
          icon: Icon(Icons.info),
          title: Text('Why RF'),
        ),
        BottomNavigationBarItem(
          icon: Icon(Icons.home),
          title: Text('Home'),
        ),
        BottomNavigationBarItem(
          icon: Icon(Icons.widgets),
          title: Text('How To'),
        ),
      ],
      currentIndex: _selectedIndex,
      selectedItemColor: Colors.blue,
      onTap: _onItemTapped,
    );
  }

  //scaff - page. outline . drawer . drawerItems
  drawerBodyItems(
    BuildContext context, {
    bool separator = false,
    FontWeight textWeight = FontWeight.w500,
    Color color = Colors.black,
    double padding = 12.0,
    TemplateRF template,
    Function onTap,
  }) {
    //template = TemplateRF.get();
    if (template.name == TemplateNameRF.modern && template.version == 'Oval') {
      separator = true;
      textWeight = FontWeight.w500;
      color = Colors.black;
      padding = 12.0;
    } else if (template.name == TemplateNameRF.modern) {
      separator = false;
      textWeight = FontWeight.w500;
      color = Colors.white;
      padding = 10.0;
    } else if (template.name == TemplateNameRF.material) {
      separator = true;
      textWeight = FontWeight.w500;
      color = Colors.black;
      padding = 12.0;
    }
    var sciStandard = [
      DrawerItemRF(
        icon: FontAwesomeIcons.userAlt,
        text: 'My Profile',
        separator: separator,
        textWeight: textWeight,
        color: Color(0xFFf77f00),
        padding: padding,
        template: _template,
        onTap: () {
          return onTap("My Profile");
        },
      ),
      DrawerItemRF(
        icon: FontAwesomeIcons.userLock,
        text: 'Change Password',
        separator: separator,
        textWeight: textWeight,
        color: Color(0xFF2a9d8f),
        padding: padding,
        template: _template,
        onTap: () {
          return onTap("My Profile");
        },
      ),
      DrawerItemRF(
        icon: FontAwesomeIcons.cloudMeatball,
        text: 'Internet Usage',
        separator: separator,
        textWeight: textWeight,
        color: Color(0xFFe63946),
        padding: padding,
        template: _template,
        onTap: () {
          rf.page.call(context, InternetUsage(), refreshCaller: true);
        },
      ),
      DrawerItemRF(
        icon: FontAwesomeIcons.database,
        text: 'Email Data Backup',
        separator: separator,
        textWeight: textWeight,
        color: Color(0xFF7209b7),
        padding: padding,
        template: _template,
        onTap: () {
          return onTap("Email Data Backup");
        },
      ),
      DrawerItemRF(
        icon: FontAwesomeIcons.solidBell,
        text: 'Notifications',
        badge: true,
        separator: separator,
        textWeight: textWeight,
        color: Color(0xFFffc300),
        padding: padding,
        template: _template,
        onTap: () {
          return onTap("Notifications");
        },
      ),
      DrawerItemRF(
        icon: FontAwesomeIcons.microphone,
        text: 'Language Setting',
        separator: separator,
        textWeight: textWeight,
        color: Color(0xFF27AE60),
        padding: padding,
        template: _template,
        onTap: () {
          rf.page.call(context, LanguageSelection(isOnboarding: false), refreshCaller: true);
        },
      ),
      DrawerItemRF(
        icon: FontAwesomeIcons.cog,
        text: 'Data Download',
        separator: separator,
        textWeight: textWeight,
        color: Color(0xFF01baef),
        padding: padding,
        template: _template,
        onTap: () {
          return onTap("Data Setting");
        },
      ),
    ];

    if (template.directives != null) if (template.directives['DrawerMenuBodySetVersion'] == 'SCI') {
      return sciStandard;
    }
    if (template.name == TemplateNameRF.modern && template.version == 'SCI') {
      return sciStandard;
    }
    if (template.name == TemplateNameRF.modern || template.name == TemplateNameRF.material) {
      return sciStandard;
    }
  }

  //scaff - page. outline . drawer . drawerFooter
  drawerFooter(
    BuildContext context, {
    bool separator = false,
    FontWeight textWeight = FontWeight.w500,
    Color color = Colors.black,
    double padding = 12.0,
    TemplateRF template,
  }) {
    template = TemplateRF.get();
    if (template.name == TemplateNameRF.modern && template.version == 'Oval') {
      separator = true;
      textWeight = FontWeight.w500;
      color = Colors.black;
      padding = 12.0;
    } else if (template.name == TemplateNameRF.modern) {
      separator = false;
      textWeight = FontWeight.w500;
      color = Colors.white;
      padding = 10.0;
    } else if (template.name == TemplateNameRF.material) {
      separator = false;
      textWeight = FontWeight.w500;
      color = Colors.pink;
      padding = 12.0;
    }
    return DrawerItemRF(
      icon: FontAwesomeIcons.headphonesAlt,
      text: 'SUPPORT',
      separator: separator,
      textWeight: textWeight,
      color: color,
      template: _template,
      padding: padding,
      onTap: () => rf.page.call(context, AuthPage()),
    );
  }
}

class ScaffoldStoreDefaultsApp {
  var validatedPop;
  var modernUI;

  ScaffoldStoreDefaultsApp(TemplateRF template) {
    validatedPop = _validatedPopFn;
    modernUI = _modernUI;
  }

  _modernUI(BuildContext context) {
    rf.page.call(context, MyApp());
  }

  _validatedPopFn(BuildContext context) {
    print('back');
    Flushbar(
      //title:  "Hey Ninja",
      messageText: Text(
        "Save change pending, do you wnat to quit?",
        style: TextStyle(
            fontWeight: FontWeight.normal, fontSize: 18.0, color: Colors.black, fontFamily: "ShadowsIntoLightTwo"),
      ),
      duration: Duration(seconds: 10),
      margin: EdgeInsets.all(20),
      borderRadius: 50,
      backgroundColor: Colors.red,
      icon: Icon(Icons.check, color: Colors.black),
      mainButton: Padding(
        padding: const EdgeInsets.only(right: 6.0),
        child: FlatButton(
          shape: new RoundedRectangleBorder(borderRadius: new BorderRadius.circular(50.0)),
          color: Colors.white,
          onPressed: () {
            Navigator.of(context).pop(true);
          },
          child: Text(
            "Quit",
            style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold, fontSize: 18.0),
          ),
        ),
      ),
    )..show(context);

//    Scaffold.of(context).showSnackBar(SnackBar(
//        content: Text("Save change pending!"),
//        action: SnackBarAction(
//          label: 'Back Anyway',
//          onPressed: () {
//            // back pop
//          },
//        )));
  }
}

class ActionItemsRF {
  ActionItemTypeRF actionType;
  Function action;
  IconData icon;

  ActionItemsRF({this.actionType, this.action, this.icon});
}

class PopupMenuItemsRF {
  IconData icon;
  String label;
  Function action;
  bool separator;

  PopupMenuItemsRF({this.icon, this.label, this.action, this.separator = true});
}
